"""Compute the spectrum of a field (fftw based)
"""
from hysop.operator.computational import Computational
from hysop.operator.discrete.spectrum import FFTSpectrum
from hysop.constants import debug
from hysop.operator.continuous import opsetup


class Spectrum(Computational):
    """
    Fourier spectrum computation of a scalar field.
    """

    def __init__(self, field, **kwds):
        """

        Parameters
        ----------
        field: :class:`~hysop.fields.continuous_field.Field`
            the input field for which spectrum will be computed
        """
        super(Spectrum, self).__init__(variables=[field], **kwds)
        self.field = field
        self.input = [field]

    def discretize(self):
        super(Spectrum, self)._fftw_discretize()

    @debug
    @opsetup
    def setup(self, rwork=None, iwork=None):
        self.discrete_op = FFTSpectrum(self.discrete_fields[self.field],
                                       method=self.method)
        nbc = self.discrete_op.res.size
        self._set_io('spectrum', (1, 2 * nbc))
        self.discrete_op.set_writer(self._writer)
        self._is_uptodate = True

    def get_work_properties(self):
        return {'rwork': None, 'iwork': None}
